#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckTrailingSpacesTabs(FilePlugin):
    name = "check_trailing_spaces_tabs"

    def run(self) -> Iterator[LinterResult]:
        """This script checks if a VT is using one or more trailing whitespaces
         or tabs.

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT
            tag_pattern: A dictionary of regex patterns that are used to find
                    tags
            special_tag_pattern: A dictionary of regex patterns that are used
                    to find special tags

        """
        spaces_tabs_matches = re.finditer(
            r"[\t ]+$", self.context.file_content, flags=re.MULTILINE
        )

        if spaces_tabs_matches:
            for spaces_tabs_match in spaces_tabs_matches:
                if (
                    spaces_tabs_match is not None
                    and spaces_tabs_match.group(0) is not None
                ):
                    yield LinterError(
                        "The VT has one or more trailing spaces and/or tabs!",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )
                    return



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugins.qod import (
    VALID_QOD_NUM_VALUES,
    VALID_QOD_TYPES,
    CheckQod,
)

from . import PluginTestCase


class CheckQodTestCase(PluginTestCase):
    nasl_file = Path("some/file.nasl")

    def test_ok_qod_num(self):
        content = '  script_tag(name:"qod", value:97);\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok_qod_type(self):
        content = '  script_tag(name:"qod_type", value:"exploit");\n'

        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

    def test_missing_qod(self):
        content = '  script_tag(name:"foo", value:"bar");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertEqual("VT is missing QoD or QoD type", results[0].message)

    def test_too_many_qod(self):
        content = (
            '  script_tag(name:"qod_type", value:"exploit");\n'
            '  script_tag(name:"qod", value:97);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertEqual("VT contains multiple QoD values", results[0].message)

    def test_wrong_qod_num_str(self):
        content = '  script_tag(name:"qod", value:"foo");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertEqual(
            'script_tag(name:"qod", value:"foo");: \'foo\' is an invalid QoD'
            " number value. Allowed are"
            f" {', '.join(str(x) for x in VALID_QOD_NUM_VALUES)}",
            results[0].message,
        )

    def test_wrong_qod_num_int(self):
        content = '  script_tag(name:"qod", value:2);\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertEqual(
            "script_tag(name:\"qod\", value:2);: '2' is an invalid QoD"
            " number value. Allowed are"
            f" {', '.join(str(x) for x in VALID_QOD_NUM_VALUES)}",
            results[0].message,
        )

    def test_wrong_qod_type(self):
        content = '  script_tag(name:"qod_type", value:"foo");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.nasl_file, file_content=content
        )
        plugin = CheckQod(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertEqual(
            'script_tag(name:"qod_type", value:"foo");: \'foo\' is an invalid'
            f" QoD type. Allowed are {', '.join(VALID_QOD_TYPES)}",
            results[0].message,
        )



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from typing import Iterator

from troubadix.helper import ScriptTag, SpecialScriptTag
from troubadix.helper.patterns import (
    _get_special_script_tag_pattern,
    get_script_tag_pattern,
)
from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckProdSvcDetectInVulnvt(FilePlugin):
    name = "check_prod_svc_detect_in_vulnvt"

    def run(self) -> Iterator[LinterResult]:
        """This script checks if the passed VT if it is doing a vulnerability
        reporting and a product / service detection together in a single VT.
        More specific this step is checking and reporting VTs having a severity
        but are placed in these Families:

        - script_family("Product detection");
        - script_family("Service detection");

        and / or are using one of the following functions:

        - register_product()
        - register_and_report_os()
        - register_and_report_cpe()
        - register_host_detail()
        - service_register()
        - service_report()
        - build_cpe()
        - build_detection_report()
        - report_host_detail_single()
        - report_host_details()
        - report_best_os_cpe()
        - report_best_os_txt()

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT
        """
        if self.context.nasl_file.suffix == ".inc":
            return

        file_content = self.context.file_content
        # Don't need to check VTs having a cvss of 0.0
        cvss_base_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_detect = cvss_base_pattern.search(file_content)

        if cvss_detect is not None and cvss_detect.group("value") == "0.0":
            return

        match_family = _get_special_script_tag_pattern(
            name=SpecialScriptTag.FAMILY.value,
            value=r"(Product|Service) detection",
        ).search(file_content)
        if match_family and match_family.group("value"):
            yield LinterError(
                "VT has a severity but is placed in the family '"
                f"{match_family.group('value')}' which is not allowed for this "
                "VT. Please split this VT into a separate Product/Service "
                "detection and Vulnerability-VT.",
                file=self.context.nasl_file,
                plugin=self.name,
            )

        matches = re.finditer(
            r"(?P<function>("
            r"register_(product|and_report_(os|cpe)|host_detail)|"
            r"service_(register|report)|"
            r"build_(cpe|detection_report)|"
            r"report_(host_(detail_single|details)|best_os_(cpe|txt)))"
            r")\s*\((?P<body>[^)]+)\)\s*;",
            file_content,
            re.MULTILINE,
        )
        if matches:
            for match in matches:
                if all(
                    det not in match.group("body")
                    for det in ["detected_by", "detected_at"]
                ):
                    yield LinterError(
                        "VT has a severity but is using the function '"
                        f"{match.group('function')}' which is not allowed for "
                        "this VT. Please split this VT into a separate "
                        "Product/Service detection and Vulnerability-VT.",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from stat import filemode
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckVTFilePermissions(FilePlugin):
    """This script checks whether the nasl file
    has the correct file permissions
    """

    name = "check_vt_file_permissions"

    def run(self) -> Iterator[LinterResult]:
        permissions = filemode(self.context.nasl_file.stat().st_mode)

        if "x" in permissions:
            yield LinterError(
                f"VT has invalid file permissions: {permissions}.\n"
                "NASL scripts must not be executable.\n"
                "Typical file permissions are '644' (-rw-r--r-) "
                "and `664` (-rw-rw-r-)",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from pathlib import Path

from troubadix.plugin import LinterError, LinterWarning
from troubadix.plugins.cve_format import CheckCVEFormat

from . import PluginTestCase


class CheckCVEFormatTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            '  script_tag(name:"cvss_base", value:"7.5");\n'
            '  script_cve_id("CVE-2022-23807");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_detection_script(self):
        path = Path("some/file.nasl")
        content = '  script_tag(name:"cvss_base", value:"0.0");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_no_cve_reference(self):
        path = Path("some/file.nasl")
        content = '  script_tag(name:"cvss_base", value:"7.5");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "VT does not refer to any CVEs.",
            results[0].message,
        )

    def test_invalid_cve_format(self):
        path = Path("some/file.nasl")
        content = (
            '  script_tag(name:"cvss_base", value:"10.0");\n'
            '  script_cve_id("CVE-a123-23807");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT uses an invalid CVE format.",
            results[0].message,
        )

    def test_more_then_four_digits(self):
        path = Path("some/file.nasl")
        content = (
            '  script_tag(name:"cvss_base", value:"7.5");\n'
            '  script_cve_id("CVE-2021-03807");\n'
        )

        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The last group of CVE digits of the VT must not "
            "start with a 0 if there are more than 4 digits.",
            results[0].message,
        )

    def test_invalid_year(self):
        path = Path("some/file.nasl")
        content = (
            '  script_tag(name:"cvss_base", value:"7.5");\n'
            '  script_cve_id("CVE-1971-3807");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT uses an invalid year in CVE format.",
            results[0].message,
        )

        current_year = datetime.now().year
        content = (
            '  script_tag(name:"cvss_base", value:"7.5");\n'
            f'  script_cve_id("CVE-{current_year + 1}-3807");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT uses an invalid year in CVE format.",
            results[0].message,
        )

    def test_duplicate_cves(self):
        path = Path("some/file.nasl")
        content = (
            '  script_tag(name:"cvss_base", value:"7.5");\n'
            '  script_cve_id("CVE-2021-3807","CVE-2021-3807");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCVEFormat(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            'VT is using CVE "CVE-2021-3807" multiple ' "times.",
            results[0].message,
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tempfile
import unittest
from datetime import datetime
from pathlib import Path

from troubadix.standalone_plugins.no_solution import (
    check_no_solutions,
    check_skip_script,
    extract_tags,
    parse_solution_date,
)


class ParseArgsTestCase(unittest.TestCase):
    def test_parse_solution_date(self):
        test_values = [
            ("01st August, 2022", datetime(2022, 8, 1)),
            ("02nd August, 2022", datetime(2022, 8, 2)),
            ("03rd August, 2022", datetime(2022, 8, 3)),
            ("04th August, 2022", datetime(2022, 8, 4)),
        ]

        for test_value, expected_result in test_values:
            result = parse_solution_date(test_value)
            self.assertEqual(result, expected_result)

    def test_check_skip_script_ok(self):
        content = (
            '  script_tag(name:"solution_type", value:"NoneAvailable");\n”'
            '  script_tag(name:"cvss_base", value:"6.4");\n'
        )

        result = check_skip_script(content)
        self.assertEqual(result, False)

    def test_check_skip_script_nok_cvss(self):
        content = (
            '  script_tag(name:"solution_type", value:"NoneAvailable");\n”'
            '  script_tag(name:"cvss_base", value:"0.0");\n'
        )

        result = check_skip_script(content)
        self.assertEqual(result, True)

    def test_check_skip_script_nok_solution_type(self):
        content = (
            '  script_tag(name:"solution_type", value:"VendorFix");\n”'
            '  script_tag(name:"cvss_base", value:"6.4");\n'
        )

        result = check_skip_script(content)
        self.assertEqual(result, True)

    def test_extract_tags_ok(self):
        content = (
            '  script_tag(name:"solution", value:"No known solution is '
            "available as of 05th August, 2022. Information regarding "
            "this issue will be updated once solution details"
            ' are available.");\n'
            '  script_tag(name:"creation_date", value:"2021-07-21 16:20:50'
            ' +0200 (Wed, 21 Jul 2021)");\n'
            '  script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
        )

        expected_result = (
            datetime(2022, 8, 5),
            datetime(2021, 7, 21),
            "1.3.6.1.4.1.25623.1.0.118132",
        )

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_extract_tags_break_solution_1(self):
        content = (
            '  script_tag(name:"creation_date", value:"2021-07-21 16:20:50'
            ' +0200 (Wed, 21 Jul 2021)");\n'
            '  script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
        )

        expected_result = None

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_extract_tags_break_solution_2(self):
        content = (
            '  script_tag(name:"solution", value:"No known solution is '
            "available as in 05th August, 2022. Information regarding "
            "this issue will be updated once solution details"
            ' are available.");\n'
            '  script_tag(name:"creation_date", value:"2021-07-21 16:20:50'
            ' +0200 (Wed, 21 Jul 2021)");\n'
            'script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
        )

        expected_result = None

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_extract_tags_break_solution_3(self):
        content = (
            '  script_tag(name:"solution", value:"No known solution is '
            "available as of 05th Imaginary, 2022. Information regarding "
            "this issue will be updated once solution details"
            ' are available.");\n'
            '  script_tag(name:"creation_date", value:"2021-07-21 16:20:50'
            ' +0200 (Wed, 21 Jul 2021)");\n'
            '  script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
        )

        expected_result = None

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_extract_tags_break_creation_date(self):
        content = (
            '  script_tag(name:"solution", value:"No known solution is '
            "available as of 05th August, 2022. Information regarding "
            "this issue will be updated once solution details are"
            ' available.");\n'
            '  script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
        )

        expected_result = None

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_extract_tags_break_oid(self):
        content = (
            '  script_tag(name:"solution", value:"No known solution is '
            "available as of 05th August, 2022. Information regarding "
            "this issue will be updated once solution details are "
            'available.");\n'
            '  script_tag(name:"creation_date", value:"2021-07-21 16:20:50'
            ' +0200 (Wed, 21 Jul 2021)");\n'
        )

        expected_result = None

        result = extract_tags(content)

        self.assertEqual(result, expected_result)

    def test_check_no_solutions(self):
        with tempfile.TemporaryDirectory() as tmp_dir:
            test_file = Path(tmp_dir, "file.txt")
            test_file.touch()

            with open(test_file, "w", encoding="LATIN-1") as file_stream:
                content = (
                    '  script_oid("1.3.6.1.4.1.25623.1.0.118132");\n'
                    '  script_tag(name:"creation_date", value:"2021-07-21 '
                    '16:20:50 +0200 (Wed, 21 Jul 2021)");\n'
                    '  script_tag(name:"cvss_base", value:"6.4");\n'
                    '  script_tag(name:"solution_type", value:"NoneAvailable");'
                    "\n"
                    '  script_tag(name:"solution", value:"No known solution'
                    " is available as of 05th August, 2022.Information "
                    "regarding this issue will be "
                    'updated once solution details are available.");\n'
                )
                file_stream.write(content)

            result = check_no_solutions([test_file], [12, 6, 1], 1)

            expected_result = [
                (
                    12,
                    [
                        (
                            test_file,
                            "1.3.6.1.4.1.25623.1.0.118132",
                            datetime(2022, 8, 5),
                            datetime(2021, 7, 21),
                        )
                    ],
                )
            ]

            self.assertEqual(result, expected_result)



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from multiprocessing import cpu_count
from pathlib import Path
from unittest.mock import Mock

from pontos.terminal import Terminal

from troubadix.argparser import parse_args


class TestArgparsing(unittest.TestCase):
    def setUp(self):
        self.terminal = Mock(spec=Terminal)

    def test_parse_files(self):
        parsed_args = parse_args(
            self.terminal,
            [
                "--files",
                "tests/plugins/test.nasl",
                "tests/plugins/fail2.nasl",
            ],
        )

        expected_files = [
            Path("tests/plugins/test.nasl"),
            Path("tests/plugins/fail2.nasl"),
        ]
        self.assertEqual(parsed_args.files, expected_files)

    def test_parse_include_tests(self):
        parsed_args = parse_args(
            self.terminal,
            [
                "--include-tests",
                "CheckBadwords",
            ],
        )
        self.assertEqual(parsed_args.included_plugins, ["CheckBadwords"])
        self.assertIsNone(parsed_args.excluded_plugins)

    def test_parse_exclude_tests(self):
        parsed_args = parse_args(
            self.terminal,
            [
                "--exclude-tests",
                "CheckBadwords",
            ],
        )
        self.assertEqual(parsed_args.excluded_plugins, ["CheckBadwords"])
        self.assertIsNone(parsed_args.included_plugins)

    def test_parse_include_patterns(self):
        parsed_args = parse_args(
            self.terminal, ["-f", "--include-patterns", "troubadix/*"]
        )

        self.assertTrue(parsed_args.full)
        self.assertEqual(parsed_args.include_patterns, ["troubadix/*"])

    def test_parse_include_patterns_fail(self):
        with self.assertRaises(SystemExit):
            parse_args(self.terminal, ["--include-patterns", "troubadix/*"])

    def test_parse_files_non_recursive_fail(self):
        with self.assertRaises(SystemExit):
            parse_args(
                self.terminal,
                [
                    "--files",
                    "tests/plugins/test.nasl",
                    "tests/plugins/fail2.nasl",
                    "--non-recursive",
                ],
            )

    def test_parse_exclude_patterns(self):
        parsed_args = parse_args(
            self.terminal, ["-f", "--exclude-patterns", "troubadix/*"]
        )

        self.assertTrue(parsed_args.full)
        self.assertEqual(parsed_args.exclude_patterns, ["troubadix/*"])

    def test_parse_max_cpu(self):
        parsed_args = parse_args(
            self.terminal,
            [
                "-f",
                "--exclude-patterns",
                "troubadix/*",
                "-j",
                "1337",
            ],
        )

        self.assertTrue(parsed_args.full)
        self.assertEqual(parsed_args.exclude_patterns, ["troubadix/*"])

        self.assertEqual(parsed_args.n_jobs, cpu_count())

    def test_parse_min_cpu(self):
        parsed_args = parse_args(
            self.terminal,
            [
                "-f",
                "--exclude-patterns",
                "troubadix/*",
                "-j",
                "-1337",
            ],
        )

        self.assertTrue(parsed_args.full)
        self.assertEqual(parsed_args.exclude_patterns, ["troubadix/*"])

        self.assertEqual(parsed_args.n_jobs, cpu_count() // 2)

    def test_parse_root(self):
        parsed_args = parse_args(self.terminal, ["--root", "foo"])

        self.assertEqual(parsed_args.root, Path("foo"))

    def test_parse_fix(self):
        parsed_args = parse_args(self.terminal, ["--fix"])

        self.assertTrue(parsed_args.fix)

    def test_parse_ignore_warnings(self):
        parsed_args = parse_args(self.terminal, ["--ignore-warnings"])

        self.assertTrue(parsed_args.ignore_warnings)

    def test_parse_log_file_statistic(self):
        parsed_args = parse_args(self.terminal, ["--log-file-statistic", "foo"])

        self.assertEqual(parsed_args.log_file_statistic, Path("foo"))



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.plugin import LinterError
from troubadix.plugins.script_xref_form import CheckScriptXrefForm


class CheckScriptXrefFormTestCase(PluginTestCase):
    path = Path("some/file.nasl")

    def test_ok(self):
        content = '  script_xref(name: "foo", value:"bar");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefForm(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckScriptXrefForm(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_wrong_name(self):
        content = '  script_xref(nammmme: "foo", value:"bar");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefForm(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            'script_xref(nammmme: "foo", value:"bar");: does not conform to'
            ' script_xref(name:"<name>", value:<value>);',
            results[0].message,
        )

    def test_wrong_value(self):
        content = '  script_xref(name: "foo", valueeeee:"bar");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefForm(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)

    def test_wrong_missing_parameters(self):
        content = '  script_xref("foo", "bar");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefForm(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from typing import Iterator

from troubadix.helper import is_ignore_file

from troubadix.plugin import FilePlugin, LinterError, LinterResult

# nb: Those are files which have this misplaced compare since their very first
# version. As changing the if now might change their behavior and the VTs are
# candidates for deprecation they are ignored for now.
IGNORE_FILES = [
    "PCIDSS_M8.2.4.a.nasl",
    "PCIDSS_M10.3.2.nasl",
    "PCIDSS_M10.2.6.nasl",
    "PCIDSS_M10.2.4.nasl",
    "PCIDSS_M8.2.5.a.nasl",
    "PCIDSS_M10.3.1.nasl",
    "PCI-DSS.nasl",
    "PCIDSS_M8.1.4.nasl",
    "PCIDSS_M10.3.3.nasl",
    "PCIDSS_M10.2.5.nasl",
    "PCIDSS_M8.1.7.nasl",
    "PCIDSS_M10.3.5.nasl",
    "PCIDSS_M8.1.8.nasl",
    "PCIDSS_M10.3.6.nasl",
    "PCIDSS_M8.2.3.a.nasl",
    "PCIDSS_M8.1.6.a.nasl",
    "PCIDSS_M10.3.4.nasl",
    "PCIDSS_M5.2.a.nasl",
    "PCIDSS_M5.1.nasl",
    "GSHB_WMI_Apache.nasl",
    "GSHB_WMI_EFS.nasl",
    "GSHB_WMI_Antivir.nasl",
    "GSHB_WMI_ProtectedMode.nasl",
    "GSHB_WMI_WinAdminTools.nasl",
    "GSHB_WMI_PolSecSet.nasl",
    "GSHB_WMI_BootDrive.nasl",
    "GSHB_WMI_W2K3_ClientFunk.nasl",
    "GSHB_WMI_IIS_Protect_SynAttack.nasl",
    "GSHB_WMI_NtpServer.nasl",
    "GSHB_WMI_IPSec_Policy.nasl",
    "GSHB_WMI_removable-media.nasl",
    "GSHB_WMI_get_AdminUsers.nasl",
    "GSHB_WMI_SNMP_Communities.nasl",
    "GSHB_WMI_IIS_UrlScanFilter.nasl",
    "GSHB_Read_Apache_Config.nasl",
    "GSHB_SMB_SDDL.nasl",
    "GSHB_WMI_IIS_RDS.nasl",
    "GSHB_WMI_PathVariables.nasl",
    "GSHB_WMI_Hibernate.nasl",
    "GSHB_Kompendium.nasl",
    "GSHB_WMI_XP-InetComm.nasl",
    "GSHB_WMI_EventLogPolSet.nasl",
    "GSHB_WMI_get_Shares.nasl",
    "GSHB_WMI_CD-FD-User-only-access.nasl",
    "GSHB_WMI_IIS_Samplefiles.nasl",
    "GSHB_WMI_list_Services.nasl",
    "GSHB_WMI_TerminalServerSettings.nasl",
    "GSHB_WMI_Driver-Autostart.nasl",
    "GSHB_WMI_Loginscreen.nasl",
    "GSHB_WMI_IIS_exec_cmd.nasl",
    "GSHB_WMI_AllowRemoteDASD.nasl",
    "GSHB_WMI_Passfilt.nasl",
    "GSHB_WMI_ScreenSaver_Status.nasl",
    "GSHB_WMI_WinFirewallStat.nasl",
    "GSHB_WMI_CD-Autostart.nasl",
    "GSHB_WMI_WIN_Subsystem.nasl",
    "GSHB_WMI_pre2000comp.nasl",
    "GSHB_WMI_PasswdPolicie.nasl",
    "GSHB_WMI_DomContrTest.nasl",
    "GSHB_WMI_get_ODBCINST.nasl",
]


class CheckMisplacedCompareInIf(FilePlugin):
    name = "check_misplaced_compare_in_if"

    def run(self) -> Iterator[LinterResult]:
        """This script checks the passed VT/Include if it is using a misplaced
            compare within an if() call like e.g.:

            if( variable >< "text" ) {}
            if( variable >< 'text' ) {}
            if( variable >!< "text" ) {}
            if( variable >!< 'text' ) {}

            instead of:

            if( "text" >< variable ) {}
            if( "text" >!< variable ) {}

        Args:
            nasl_file: The VT/Include that is going to be checked
            file_content: The content of the VT
        """

        if is_ignore_file(self.context.nasl_file, IGNORE_FILES):
            return

        # pylint: disable=W0511
        # TODO: Find a better way to parse if calls as this would miss
        #  something like e.g.:
        #
        # if((foo =~ "bar || bar =~ "foo") || foobar = "foo"){}
        #
        # nb: We can't use { as an ending delimiter as there could be also
        #  something like e.g.:
        #
        # if((foo =~ "bar || bar =~ "foo") || foobar = "foo")
        #   bar = "foo"; (no ending {)
        # maybe this regex fixes this:
        #   r"^\s*(if|}?\s*else if)\s*\((?P<condition>.*)\)\s*({|(.*|.*\n.*);)"
        # original regex:
        #   r"^\s*(if|}?\s*else if)\s*\(([^)]+)"
        if_matches = re.finditer(
            r"^\s*(if|}?\s*else if)\s*\((?P<condition>.*)\)\s*({|(.*|.*\n.*);)",
            self.context.file_content,
            re.MULTILINE,
        )

        if not if_matches:
            return

        for if_match in if_matches:
            if if_match:
                misplaced_compare_match = re.search(
                    r"((if|}?\s*else if)\s*\("
                    r"\s*|\|\|\s*|&&\s*)["
                    r"a-zA-Z_]+\s*>\!?<\s*("
                    r'"|\')',
                    if_match.group(0),
                )
                if misplaced_compare_match:
                    yield LinterError(
                        f"VT/Include is using a misplaced compare "
                        f"within an if() call in {if_match.group(0)}",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Main module for troubadix """

import sys
from pathlib import Path
from typing import Iterable, List, Tuple

from pontos.terminal import Terminal
from pontos.terminal.terminal import ConsoleTerminal

from troubadix.__version__ import __version__
from troubadix.argparser import parse_args
from troubadix.helper import get_root
from troubadix.reporter import Reporter
from troubadix.runner import Runner


def generate_file_list(
    dirs: Iterable[Path],
    exclude_patterns: Iterable[str],
    include_patterns: Iterable[str],
) -> List[Path]:
    """Generates a files list under respect of several given arguments

    Arguments:
    dirs                List of dirs, within looking for files
    exclude_patterns    List of glob patterns,
                        exclude files that fit the pattern
    include_patterns    List of glob patterns,
                        include files that fit the pattern
                        with respect of excluded files

    Returns
    List of Path objects"""
    files: List[Path] = []
    for directory in dirs:
        for pattern in include_patterns:
            files.extend(directory.glob(pattern))
    if exclude_patterns:
        excluded_files = []
        for directory in dirs:
            for pattern in exclude_patterns:
                excluded_files.extend(directory.glob(pattern))
        files = [f for f in files if f not in excluded_files]

    return files


def generate_patterns(
    terminal: ConsoleTerminal,
    include_patterns: List[str],
    exclude_patterns: List[str],
    non_recursive: bool,
) -> Tuple[List[str], List[str]]:
    """Generates the include and exclude patterns

    Arguments:
        include_patterns:    List of glob patterns to filter files with
        exclude_patterns:    List of glob patterns of files that will excluded
        non_recursive:       Whether to include all sub directories to the
                            patterns or not

    Returns:
        Corrected tuple of lists of include and exclude patterns
    """
    # Setting the globs for include all nasl and inc files, if no include
    # pattern given
    if not include_patterns:
        include_patterns = ["*.nasl", "*.inc"]

    if not non_recursive:
        # if non-recursive (default), add recursive pattern to globs
        include_patterns = [f"**/{pattern}" for pattern in include_patterns]
        if exclude_patterns:
            exclude_patterns = [f"**/{pattern}" for pattern in exclude_patterns]
    else:
        terminal.warning("Running in non-recursive mode!")

    return include_patterns, exclude_patterns


def from_file(include_file: Path, term: Terminal) -> Iterable[Path]:
    """Parse the given file containing a list of files into"""
    try:
        return [
            Path(f)
            for f in include_file.read_text(encoding="utf-8").splitlines()
        ]
    except FileNotFoundError:
        term.error(f"File {include_file} containing the file list not found.")
        sys.exit(1)


def main(args=None):
    """Main process of greenbone-docker"""
    term = ConsoleTerminal()

    if not args:
        args = sys.argv[1:]

    parsed_args = parse_args(terminal=term, args=args)

    if parsed_args.version:
        term.info(f"troubadix version {__version__}")
        sys.exit(1)

    # Full will run in the root directory of executing. (Like pwd)
    if parsed_args.full:
        cwd = Path.cwd()
        dirs = [cwd]
        term.info(f"Running full lint from {cwd}")
    else:
        dirs = parsed_args.dirs

    files = None
    if dirs:
        include_patterns, exclude_patterns = generate_patterns(
            terminal=term,
            include_patterns=parsed_args.include_patterns,
            exclude_patterns=parsed_args.exclude_patterns,
            non_recursive=parsed_args.non_recursive,
        )

        files = generate_file_list(
            dirs=dirs,
            exclude_patterns=exclude_patterns,
            include_patterns=include_patterns,
        )

    elif parsed_args.from_file:
        files = from_file(include_file=parsed_args.from_file, term=term)

    elif parsed_args.files:
        files = parsed_args.files

    if not files:
        term.warning("No files given/found.")
        sys.exit(1)

    # Remove duplicate files
    files = list(set(files))

    # Get the root of the nasl files
    if parsed_args.root:
        root = parsed_args.root
    else:
        first_file = files[0].resolve()
        root = get_root(first_file)

    reporter = Reporter(
        term=term,
        fix=parsed_args.fix,
        log_file=parsed_args.log_file,
        log_file_statistic=parsed_args.log_file_statistic,
        root=root,
        statistic=True if not parsed_args.no_statistic else False,
        verbose=parsed_args.verbose,
        ignore_warnings=parsed_args.ignore_warnings,
    )

    runner = Runner(
        reporter=reporter,
        n_jobs=parsed_args.n_jobs,
        excluded_plugins=parsed_args.excluded_plugins,
        included_plugins=parsed_args.included_plugins,
        fix=parsed_args.fix,
        ignore_warnings=parsed_args.ignore_warnings,
        root=root,
    )

    term.info(f"Start linting {len(files)} files ... ")

    # Return exit with 1 if error exist
    if not runner.run(files):
        sys.exit(1)


if __name__ == "__main__":
    main()



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from argparse import ArgumentError
from unittest import TestCase

from troubadix.standalone_plugins.changed_packages.package import (
    Package,
    Reasons,
)


class ReasonsTextCase(TestCase):
    def test_from_cli_argument(self):
        self.assertEqual(
            Reasons.DROPPED_ARCHITECTURE,
            Reasons.from_cli_argument("dropped-architecture"),
        )

    def test_from_cli_argument_not_okay(self):
        with self.assertRaises(ArgumentError):
            Reasons.from_cli_argument("foo")

    def test_str(self):
        self.assertEqual(str(Reasons.ADDED_EPOCH), "added-epoch")


class PackageTestCase(TestCase):
    def test_lt_by_name(self):
        package = Package("a-foo", "1.2.3", "DEB11")
        other_package = Package("b-foo", "1.2.3", "DEB11")

        self.assertLess(package, other_package)

    def test_lt_by_version(self):
        package = Package("foo", "1.2.3", "DEB11")
        other_package = Package("foo", "2.2.3", "DEB11")

        self.assertLess(package, other_package)

    def test_lt_by_release(self):
        package = Package("foo", "1.2.3", "DEB10")
        other_package = Package("foo", "1.2.3", "DEB11")

        self.assertLess(package, other_package)

    def test_lt_nothing_in_common(self):
        package = Package("foo", "1.2.3", "DEB10")
        other_package = Package("foo", "1.2.3", "DEB10")

        self.assertFalse(package < other_package)
        self.assertFalse(package > other_package)

    def test_lt_release_first(self):
        package = Package("b", "1.2.3", "DEB10")
        other_package = Package("a", "1.2.3", "DEB11")

        self.assertLess(package, other_package)
        self.assertFalse(other_package < package)
